/*
  This program lets you run multiple instances of computeProgram_64.exe.

  You can watch how Windows schedules those processes among the CPU's.

  Use the Windows Task manager to restrict the processor affinity of
  the cmd.exe shell that you use to run this program. Notice how
  Windows will only schedule the child processes among the assigned
  CPU's.
*/
#include <windows.h>
#include <stdio.h>

/* function prototype */
void printError(char* functionName);

int main( int argc, char *argv[])
{
   int i;
   char commandLine[100];
   PROCESS_INFORMATION *processInfo = malloc( (argc-1) * sizeof(PROCESS_INFORMATION) );

   STARTUPINFO startInfo;
   ZeroMemory(&startInfo, sizeof(startInfo));
   startInfo.cb = sizeof(startInfo);

   if (argc < 2)
   {
      fprintf(stderr, "usage, %s  SECONDS...\n", argv[0]);
      return 0;
   }

   for (i = 1; i < argc; i++)
   {
      /* set up the command line */
      sprintf(commandLine, "%s", "computeProgram_64.exe");
      /* put a command-line argument from this process on the command-line of the child process.*/
      sprintf(commandLine, "%s %s", commandLine, argv[i]);

      /* start the child process */
      if( !CreateProcess(NULL, commandLine, NULL, NULL, TRUE,
                         NORMAL_PRIORITY_CLASS | CREATE_NEW_CONSOLE,
                         NULL, NULL, &startInfo, &processInfo[i-1]) )
      {
          printError("CreateProcess");
      }
   }

   /* wait for each child process to end */
   for (i = 0; i < argc; i++)
   {
      WaitForSingleObject(processInfo[i].hProcess, INFINITE);
   }

   for (i = 0; i < argc; i++)
   {
      /* close all the handles */
      CloseHandle(processInfo[i].hThread);
      CloseHandle(processInfo[i].hProcess);
   }

   return 0;
}



/****************************************************************
   The following function can be used to print out "meaningful"
   error messages. If you call a Win32 function and it returns
   with an error condition, then call this function right away and
   pass it a string containing the name of the Win32 function that
   failed. This function will print out a reasonable text message
   explaining the error and then (if chosen) terminate the program.
*/
void printError(char* functionName)
{
   LPVOID lpMsgBuf;
    int error_no;
    error_no = GetLastError();
    FormatMessage(
         FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
         NULL,
         error_no,
         MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), /* Default language */
         (LPTSTR) &lpMsgBuf,
         0,
         NULL
    );
    /* Display the string. */
    fprintf(stderr, "\n%s failed on error %d: ", functionName, error_no);
    fprintf(stderr, (const char*)lpMsgBuf);
    /* Free the buffer. */
    LocalFree( lpMsgBuf );
}
